// Functions for easy use of audio player and microphone on Kamami KAMeLeon STM32L496 board.
// jk 12'2017
// The initialization code was generated in STM32CubeMX.

#ifndef	_KAMAMI_L496_AUDIO_H
#define	_KAMAMI_L496_AUDIO_H
#include <stdint.h>

/* Initialize audio player and microphone. 
 * The function should be called before using any other audio function. 
 * It initializes GPIO lines, timer, ADC and DAC for audio record and playback. */
void audio_init(void);

/* Start recording audio by storing audio samples to the provided buffer. 
 * The buffer may be handled by providing implementation of the functions:
 * void HAL_ADC_ConvHalfCpltCallback(ADC_HandleTypeDef* hadc) and
 * void HAL_ADC_ConvCpltCallback(ADC_HandleTypeDef* hadc),
 * which are called once half and all of the buffer is filled respectively.
 * @param buf: buffer for storing audio samples (used as circular DMA buffer) 
 * @param len: length of the buffer (in bytes)
 */
void audio_start_rec(uint8_t* buf, uint32_t len);

/* Stop audio recording. */
void audio_stop_rec(void);

/* Start playing audio from the provided buffer. 
 * The filling of the buffer may be handled by providing implementation of the functions:
 * void HAL_DAC_ConvHalfCpltCallbackCh1(DAC_HandleTypeDef* hdac) and
 * void HAL_DAC_ConvCpltCallbackCh1(DAC_HandleTypeDef* hdac),
 * which are called once half and all of the buffer is filled respectively.
 * @param buf: buffer with audio samples (used as circular DMA buffer) 
 * @param len: length of the buffer (in bytes)
 */
void audio_start_play(uint8_t* buf, uint32_t len);

/* Stop playing audio. */
void audio_stop_play(void);

#endif
