// Functions for easy use of LM75 temperature sensor on Kamami KAMeLeon STM32L496 board.
// jk 11'2017
// The initialization code was generated in STM32CubeMX.

#include "kamami_l496_temp.h"
#include "stm32l4xx_hal.h"

#define TEMP_LM75_I2C1_SDA_Pin GPIO_PIN_13
#define TEMP_LM75_I2C1_SDA_GPIO_Port GPIOG
#define TEMP_LM75_I2C1_SCL_Pin GPIO_PIN_14
#define TEMP_LM75_I2C1_SCL_GPIO_Port GPIOG
#define TEMP_LM75_INT_Pin GPIO_PIN_15
#define TEMP_LM75_INT_GPIO_Port GPIOG

#define TEMP_LM75_TEMP_REG	0
#define TEMP_LM75_CONF_REG	1
#define TEMP_LM75_THYS_REG	2
#define TEMP_LM75_TOS_REG	3

#define	KAMAMI_L496_TEMP_LM75_ADDR	0x48

I2C_HandleTypeDef hi2c1;

static void MX_I2C1_Init(void);

void temp_lm75_init(void)
{
  // configure clock for I2C1
  RCC_PeriphCLKInitTypeDef PeriphClkInit;
  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_I2C1;
  PeriphClkInit.I2c1ClockSelection = RCC_I2C1CLKSOURCE_PCLK1;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }	
   // configure GPIO lines for LM75
    GPIO_InitTypeDef GPIO_InitStruct;

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOG_CLK_ENABLE();
  HAL_PWREx_EnableVddIO2();

  /*Configure GPIO pin : TEMP_LM75_INT_Pin */
  GPIO_InitStruct.Pin = TEMP_LM75_INT_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(TEMP_LM75_INT_GPIO_Port, &GPIO_InitStruct);
  
  /**I2C1 GPIO Configuration    
  PG13     ------> I2C1_SDA
  PG14     ------> I2C1_SCL 
  */
  GPIO_InitStruct.Pin = TEMP_LM75_I2C1_SDA_Pin|TEMP_LM75_I2C1_SCL_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF4_I2C1;
  HAL_GPIO_Init(GPIOG, &GPIO_InitStruct);
  
  /* Peripheral clock enable */
  __HAL_RCC_I2C1_CLK_ENABLE();  

  MX_I2C1_Init();
}

static uint8_t read_8b_reg(uint8_t reg_addr)
{
  uint8_t temp;
  if(HAL_I2C_Mem_Read(&hi2c1, KAMAMI_L496_TEMP_LM75_ADDR << 1, reg_addr, 1, (uint8_t*)&temp, 1, 1000) != HAL_OK)
	  _Error_Handler(__FILE__, __LINE__);
  return temp;
}

// read temperature in degrees Celsius
int8_t temp_lm75_read(void)
{
	return (int8_t)read_8b_reg(TEMP_LM75_TEMP_REG);
}

/* I2C1 init function */
static void MX_I2C1_Init(void)
{

  hi2c1.Instance = I2C1;
  hi2c1.Init.Timing = 0x10909CEC;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.OwnAddress2Masks = I2C_OA2_NOMASK;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c1) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Configure Analogue filter 
    */
  if (HAL_I2CEx_ConfigAnalogFilter(&hi2c1, I2C_ANALOGFILTER_ENABLE) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Configure Digital filter 
    */
  if (HAL_I2CEx_ConfigDigitalFilter(&hi2c1, 0) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

}
